#include "esp_camera.h"
#include <WiFi.h>
#include "IFTTTWebhook.h"
#define IFTTT_API_KEY "YOUR IFTTT API KEY HERE"
#define IFTTT_EVENT_NAME "YOUR IFTTT EVENT NAME HERE"
IFTTTWebhook wh(IFTTT_API_KEY, IFTTT_EVENT_NAME);
int btn = 2;
int floodlight = 4;// external LED alone on P12 not bright enough
char curFlood;
long thresh = 200;
long lastPress;
long actDelay = 20;// delay in minutes
long actThresh = 0;// will be the ongoing actDelay after first activation
long lastAct;
int btnVal;
boolean canSend = true;
int getFlood();// forward declaration of floodlight status method in app_httpd class
#define CAMERA_MODEL_AI_THINKER

#include "camera_pins.h"

const char* ssid = "YOUR NETWORK SSID HERE";
const char* password = "YOUR NETWORK PASSWORD HERE";

void startCameraServer();

void setup() {
  Serial.begin(115200);
  pinMode(btn, INPUT);
  pinMode(floodlight, OUTPUT);
  pinMode(33, OUTPUT);
  Serial.setDebugOutput(false);
  Serial.println();

  camera_config_t config;
  config.ledc_channel = LEDC_CHANNEL_0;
  config.ledc_timer = LEDC_TIMER_0;
  config.pin_d0 = Y2_GPIO_NUM;
  config.pin_d1 = Y3_GPIO_NUM;
  config.pin_d2 = Y4_GPIO_NUM;
  config.pin_d3 = Y5_GPIO_NUM;
  config.pin_d4 = Y6_GPIO_NUM;
  config.pin_d5 = Y7_GPIO_NUM;
  config.pin_d6 = Y8_GPIO_NUM;
  config.pin_d7 = Y9_GPIO_NUM;
  config.pin_xclk = XCLK_GPIO_NUM;
  config.pin_pclk = PCLK_GPIO_NUM;
  config.pin_vsync = VSYNC_GPIO_NUM;
  config.pin_href = HREF_GPIO_NUM;
  config.pin_sscb_sda = SIOD_GPIO_NUM;
  config.pin_sscb_scl = SIOC_GPIO_NUM;
  config.pin_pwdn = PWDN_GPIO_NUM;
  config.pin_reset = RESET_GPIO_NUM;
  config.xclk_freq_hz = 20000000;
  config.pixel_format = PIXFORMAT_JPEG;
  //init with high specs to pre-allocate larger buffers
  if (psramFound()) {
    config.frame_size = FRAMESIZE_UXGA;
    config.jpeg_quality = 10;
    config.fb_count = 2;
  } else {
    config.frame_size = FRAMESIZE_SVGA;
    config.jpeg_quality = 12;
    config.fb_count = 1;
  }


  // camera init
  esp_err_t err = esp_camera_init(&config);
  if (err != ESP_OK) {
    Serial.printf("Camera init failed with error 0x%x", err);
    return;
  }

  sensor_t * s = esp_camera_sensor_get();
  s->set_awb_gain(s, 0);// start floodlight slider as OFF
  s->set_whitebal(s, 0); // start auto white balance slider as OFF
  //initial sensors are flipped vertically and colors are a bit saturated
  if (s->id.PID == OV3660_PID) {
    s->set_vflip(s, 1);//flip it back
    s->set_brightness(s, 1);//up the blightness just a bit
    s->set_saturation(s, -2);//lower the saturation
  }
  //drop down frame size for higher initial frame rate
  s->set_framesize(s, FRAMESIZE_QVGA);// was QVGA

#if defined(CAMERA_MODEL_M5STACK_WIDE)
  s->set_vflip(s, 1);
  s->set_hmirror(s, 1);
#endif

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");

  startCameraServer();

  Serial.print("Camera Ready! Use 'http://");
  Serial.print(WiFi.localIP());
  Serial.println("' to connect");
  lastPress = millis();
  lastAct = millis();
  digitalWrite(33, LOW);
  delay(500);
  digitalWrite(33, HIGH);


  Serial.println("setup done.");
}

void loop() {
  curFlood = getFlood();
  if (curFlood == 3) {// turn floodlight ON
    //Serial.println("ON");
    digitalWrite(floodlight, HIGH);
  } else {
    digitalWrite(floodlight, LOW);// turn floodlight OFF
    //Serial.println("   OFF");
  }

  btnVal = digitalRead(btn);// look for end stop button press indication job done
  if (btnVal == HIGH && millis() - lastPress > thresh) {
    Serial.print(btnVal);
    Serial.println("   *** button press ***");
    wh.trigger();
    delay(2500);
    lastPress = millis();// reset button watche

  }
}
